@@ -114,29 +114,6 @@ class TestHipparcosRereductionDVDBook:
         assert np.isclose(data._epoch[84], 1991.952)
         assert np.isclose(np.sin(data.scan_angle[84]), -0.8083, rtol=.01)
 
-    def test_reject_obs(self):
-        test_data_directory = os.path.join(os.getcwd(), 'htof/test/data_for_tests/Hip2')
-        data = HipparcosRereductionDVDBook()
-        data.parse(star_id='27321', intermediate_data_directory=test_data_directory)
-        assert len(data) == 111 - 0
-        assert data.rejected_epochs == []
-
-        data.parse(star_id='84', intermediate_data_directory=test_data_directory)
-        assert len(data) == 96 - 1
-        assert np.allclose(np.sort(data.rejected_epochs), [69])
-
-        data.parse(star_id='70', intermediate_data_directory=test_data_directory)
-        assert len(data) == 112 - 5
-        assert np.allclose(np.sort(data.rejected_epochs), [36, 55, 59, 64, 105])
-
-        data.parse(star_id='40', intermediate_data_directory=test_data_directory)
-        assert len(data) == 134 - 2
-        assert np.allclose(np.sort(data.rejected_epochs), [6, 53])
-
-        data.parse(star_id='072477', intermediate_data_directory=test_data_directory)
-        assert len(data) == 64 - 0
-        assert data.rejected_epochs == []
-
 
 class TestHipparcosRereductionJavaTool:
     test_data_directory = os.path.join(os.getcwd(), 'htof/test/data_for_tests/Hip21')
@@ -144,21 +121,78 @@ class TestHipparcosRereductionJavaTool:
     def test_parse(self):
         data = HipparcosRereductionJavaTool()
         data.parse(star_id='27321', intermediate_data_directory=self.test_data_directory)
-        u = 1  # Error inflation factor
+        u = 0.875291  # D. Michalik et al. 2014 Q factor for Hip 27321, calculated by hand
         assert len(data) == 111
+        assert data.meta['star_id'] == '27321'
+        assert data.meta['catalog_f2'] == -1.63
+        assert data.meta['catalog_soltype'] == 5
+        assert np.isclose(data.meta['calculated_f2'], -1.64, atol=0.01)
         assert np.isclose(data._epoch[0], 1990.0055)
         assert np.isclose(np.sin(data.scan_angle[0]), -0.9050, rtol=.01)
-        assert np.isclose(data.along_scan_errs.values[0], 0.80 * u)
+        assert np.isclose(data.along_scan_errs.values[0], 0.80 * u, atol=0.01)
         assert np.isclose(data._epoch[84], 1991.9523)
         assert np.isclose(np.sin(data.scan_angle[84]), -0.8083, rtol=.01)
 
-    def test_outlier_reject(self):
+    def test_outlier_reject_nowriteoutbug(self):
+        # test that outliers are rejected when it is a source without the data corruption.
         data = HipparcosRereductionJavaTool()
         data.parse(star_id='27100', intermediate_data_directory=self.test_data_directory)
         assert len(data) == 147 - 2  # num entries - num outliers
         # outliers are marked with negative AL errors. Assert outliers are gone.
         assert np.all(data.along_scan_errs > 0)
 
+    def test_outlier_reject_warning(self):
+        # test that outlier reject throws a warning
+        data = HipparcosRereductionJavaTool()
+        data.parse(star_id='4427', intermediate_data_directory=self.test_data_directory)
+        assert True
+
+    @pytest.mark.filterwarnings("ignore:attempt_adhoc_rejection")
+    @pytest.mark.parametrize("hip_id,rej_obs", [('39', {'orbit/scan_angle/time': [75],
+                                                        'residual/along_scan_error': [113]}),
+                                                ('651', {'orbit/scan_angle/time': [98],
+                                                         'residual/along_scan_error': [164]}),
+                                                ('94046', {'orbit/scan_angle/time': [63, 65],
+                                                           'residual/along_scan_error': [77, 76]}),
+                                                ('44050', {'orbit/scan_angle/time': [8, 11, 44],
+                                                           'residual/along_scan_error': [73, 72, 71]}),
+                                                ('114114', {'orbit/scan_angle/time': [1, 17, 22, 56],
+                                                           'residual/along_scan_error': [79, 80, 81, 82]}),
+                                                ('27321', {}),
+                                                ('072477', {})])
+    def test_reject_obs(self, hip_id, rej_obs):
+        # hip 651 is a great test source because it has 3 rejected observations (negative AL errors)
+        # and it has an uncatalogued rejection that we need to fix (i.e., an extra, 4th, rejection).
+        test_data_directory = os.path.join(os.getcwd(), 'htof/test/data_for_tests/Hip21')
+        data = HipparcosRereductionJavaTool()
+        # get info on the IAD without doing any rejection:
+        data.parse(star_id=hip_id, intermediate_data_directory=test_data_directory,
+                   attempt_adhoc_rejection=False, reject_known=False)
+        nobs_initial = len(data)
+        print(hip_id, nobs_initial)
+        num_known_rejects = np.count_nonzero(data.along_scan_errs.values < 0)
+        # now run the rejection routine and see how it compares to what we expect:
+        data.parse(star_id=hip_id, intermediate_data_directory=test_data_directory)
+        nobs_after_rejection = len(data)
+        num_rejects_writeout_bug = len(rej_obs.get('orbit/scan_angle/time', []))
+        assert nobs_after_rejection == nobs_initial - num_rejects_writeout_bug - num_known_rejects
+        # Note that for hip39, any of the orbits within 1426 are ok to reject. I.e. 70, 71, 72, 73, 74, 75.
+        sum_chi2_partials = calculate_chisq_partials(data)
+        assert sum_chi2_partials < 0.12  # assert that the IAD reflect a solution that is a stationary point
+        if len(rej_obs) > 0:
+            assert np.allclose(np.sort(data.additional_rejected_epochs['orbit/scan_angle/time']),
+                               np.sort(rej_obs['orbit/scan_angle/time']))
+            assert np.allclose(np.sort(data.additional_rejected_epochs['residual/along_scan_error']),
+                               np.sort(rej_obs['residual/along_scan_error']))
+
+    @pytest.mark.parametrize("hip_id", [17447, 21000, 94312])
+    def test_reject_obs_from_precomputed_list(self, hip_id):
+        test_data_directory = os.path.join(os.getcwd(), 'htof/test/data_for_tests/Hip21')
+        data = HipparcosRereductionJavaTool()
+        data.parse(star_id=hip_id, intermediate_data_directory=test_data_directory)
+        sum_chi2_partials = calculate_chisq_partials(data)
+        assert sum_chi2_partials < 0.12  # assert that the IAD reflect a solution that is a stationary point
+
 
 class TestDataParser:
     def test_parse_raises_file_not_found_error(self):
@@ -211,6 +245,11 @@ class TestDataParser:
         assert len(DataParser()) == 0
         assert len(DataParser(epoch=pd.DataFrame(np.arange(4)))) == 4
 
+    def test_access_rejected_epochs(self):
+        data = HipparcosRereductionDVDBook()
+        data._rejected_epochs = 'test'
+        assert data.rejected_epochs == 'test'
+
 
 def test_convert_dates_to_jd():
     epoch = pd.DataFrame(data=[1990.0, 1990.25], index=[5, 6])
@@ -416,3 +455,17 @@ def test_get_nparam():
     assert 5 == get_nparam('23115')
     assert 5 == get_nparam('95')
     assert 7 == get_nparam('97')
+
+
+def calculate_chisq_partials(data: DataParser):
+    sin_scan = np.sin(data.scan_angle.values)
+    cos_scan = np.cos(data.scan_angle.values)
+    dt = data.epoch - 1991.25
+    orbit_factors = np.array([sin_scan, cos_scan, dt * sin_scan, dt * cos_scan])
+    # this simultaneously deletes one of the residuals, assigns the remaining residuals to the
+    # shifted orbits, and calculates the chi2 partials vector per orbit:
+    residual_factors = (data.residuals.values / data.along_scan_errs.values ** 2)
+    chi2_vector = (2 * residual_factors * orbit_factors).T
+    # sum the square of the chi2 partials to decide for whether or not it is a stationary point.
+    sum_chisquared_partials = np.sqrt(np.sum(np.sum(chi2_vector,axis=0) ** 2))
+    return sum_chisquared_partials
